/*-------------------------------------------------------------------------
|   A wrapper to convert RXTX into Linux Java Comm
|   Copyright 1998 Kevin Hester, kevinh@acm.org
|
|   This library is free software; you can redistribute it and/or
|   modify it under the terms of the GNU Library General Public
|   License as published by the Free Software Foundation; either
|   version 2 of the License, or (at your option) any later version.
|
|   This library is distributed in the hope that it will be useful,
|   but WITHOUT ANY WARRANTY; without even the implied warranty of
|   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
|   Library General Public License for more details.
|
|   You should have received a copy of the GNU Library General Public
|   License along with this library; if not, write to the Free
|   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--------------------------------------------------------------------------*/
package gnu.io;

import javax.comm.*;

import java.io.*;
import java.util.*;

class RXTXPort
extends SerialPort
    {
	private final static boolean debug = false;

	/** Open the named port */
	public RXTXPort(String name) throws IOException {
		if( debug ) {
			System.out.println( "RXTXPort( \"" + name + "\" )" );
		}
		this.name = name;
		out = new SerialOutputStream(name);
		in = new SerialInputStream(name);
    		out_termios = new Termios(out.PortNumber);
    		in_termios = new Termios(in.PortNumber);
    }
        
    /*
     * Methods to implement CommPort
     */

	public InputStream getInputStream() throws IOException
	{
		return in;
	}

	public OutputStream getOutputStream() throws IOException
	{
		return out;
	}

	public void close()
	{
		if( debug ) {
			System.out.println( "RXTXPort( close )");
		}
		if (this.SPEventListener != null) {
			monitor = 0;
			this.thread_die = true;
			this.SPEventListener = null;
		}
		try {
			in.close();
			out.close();
		} catch(IOException e) { }
		super.close();
	}

	public int catchsigio()
	{
		return(in.catchsigio());
	}
	public void enableReceiveThreshold(int thresh)
	{
		in.enablereceivethreshold(in.PortNumber, thresh);
	}

	public void disableReceiveThreshold()
	{
		in.disablereceivethreshold(in.PortNumber);
	}

	public boolean isReceiveThresholdEnabled()
	{
		return(in.isreceivethresholdenabled(in.PortNumber));
	}

	public int getReceiveThreshold()
	{
		return(in.getreceivethreshold(in.PortNumber));
	}

	public void enableReceiveTimeout(int rcvTimeout)
	{
		in.noblock(in.PortNumber, BLOCK);
		in.settimeout(in.PortNumber, rcvTimeout);
		noblock = false;
	}

	public void disableReceiveTimeout()
	{
		in.noblock(in.PortNumber, NOBLOCK);
		noblock = true;
	}

	public boolean isReceiveTimeoutEnabled()

	{
		return (!noblock);
	}

	public int getReceiveTimeout()
	{
		return (in.gettimeout(in.PortNumber));
	}

	public void enableReceiveFraming(int framingByte)
	{
		in.enablereceiveframing(in.PortNumber, framingByte);
	}

	public void disableReceiveFraming()
	{
		in.disablereceiveframing(in.PortNumber);
	}

	public boolean isReceiveFramingEnabled()
	{
		return(in.isreceiveframingenabled(in.PortNumber));
	}

	public int getReceiveFramingByte()
	{
		return(in.getreceiveframingbyte(in.PortNumber));
	}

	public void setInputBufferSize(int size)
	{
		in.setinputbuffersize(in.PortNumber,size);
	}
	
	public int getInputBufferSize()
	{
		return(in.getinputbuffersize(in.PortNumber));
	}

	public void setOutputBufferSize(int size)
	{
		out.setoutputbuffersize(out.PortNumber, size);
	}

	public int getOutputBufferSize()
	{
		return(out.getoutputbuffersize(out.PortNumber));
	}
	
	/*
	Methods to implement SerialPort
	*/
    
	public int getBaudRate()
	{
		try
			{
				return in.GetSpeed(in.PortNumber);
			}
		catch(IOException e)
			{
				return 0;
			}
	}

	public int getDataBits()
	{
	return databits;
	}

	public int getStopBits()
	{
		return stopbits;
	}

	public int getParity()
	{
		return parity;
	}

	public void sendBreak(int millis)
	{
		// tcsendbreak sends for .25s for every unit passed in
		out.tcsendbreak(out.PortNumber, millis / 250);
	}

	public void setFlowControlMode(int flowcontrol)
	{
		int oldflow = flowmode;

		flowmode = flowcontrol;

		if (flowmode == FLOWCONTROL_RTSCTS_IN)
			in.setHWFC(in.PortNumber, 1); // On
		else
			in.setHWFC(in.PortNumber, 0); // Off
	/*
		try {
			updatePortParams();
		}
		catch(UnsupportedCommOperationException e) {
			flowmode = oldflow;  // something failed
		} 
	*/
	}


	public int getFlowControlMode()
	{
		return flowmode;
	}

	/**
	   Set the Linux flowmode, databits, stopbits, parity
	*/
	private void updatePortParams() throws UnsupportedCommOperationException
	{
		in_termios.filltermios(in.PortNumber); out_termios.filltermios(out.PortNumber);
		in_termios.c_cflag = translate_dataBits(in_termios.c_cflag, databits);
		in_termios.c_cflag = translate_stopBits(in_termios.c_cflag, stopbits);
		in_termios.c_cflag = translate_parity(in_termios.c_cflag, parity);
		out_termios.c_cflag = translate_dataBits(out_termios.c_cflag, databits);
		out_termios.c_cflag = translate_stopBits(out_termios.c_cflag, stopbits);
		out_termios.c_cflag = translate_parity(out_termios.c_cflag, parity);
		in_termios.dumptermios(in.PortNumber);
		out_termios.dumptermios(out.PortNumber);
		out.tcsetattr(out.PortNumber, in_termios.jstring2termios("TCAFLUSH"));
		in.tcsetattr(out.PortNumber, in_termios.jstring2termios("TCAFLUSH"));
		}


	public void setSerialPortParams(int baudrate,
					int dataBits,
					int stopBits,
					int parity) 
		    throws UnsupportedCommOperationException
	{
		try
			{
			in.SetSpeed(in.PortNumber, baudrate);
			out.SetSpeed(in.PortNumber, baudrate);

			databits = dataBits;
			stopbits = stopBits;
			this.parity = parity;
			updatePortParams();
		} catch(IOException e) { }
	}

	private int translate_parity(int flag, int bits)
	{
		int Parenb=in_termios.jstring2termios("PARENB");
		int Parodd=in_termios.jstring2termios("PARODD");

		if (bits == SerialPort.PARITY_NONE)
			return(flag & ~Parenb);
		if (bits == SerialPort.PARITY_EVEN)
			return(flag | Parenb & ~Parodd);
		return(flag | Parenb | Parodd);
	}

	private int translate_stopBits(int flag, int bits)
	{
		int Cstopb=in_termios.jstring2termios("CSTOPB");

		if (bits == SerialPort.STOPBITS_1)
			return(flag & ~Cstopb);
		return(flag | Cstopb);
	}

	private int translate_dataBits(int flag, int bits)
	{
		int Result=flag & ~in_termios.jstring2termios("CSIZE");

		if (bits == SerialPort.DATABITS_5)
			return(Result | in_termios.jstring2termios("CS5"));
		if (bits == SerialPort.DATABITS_6)
			return(Result | in_termios.jstring2termios("CS6"));
		if (bits == SerialPort.DATABITS_7)
			return(Result | in_termios.jstring2termios("CS7"));
		if (bits == SerialPort.DATABITS_8)
			return(Result| in_termios.jstring2termios("CS8")); 
		return(flag);

	}

	public void setDTR(boolean dtr)
	{
		in.setDTR(in.PortNumber, dtr);
	}

	public boolean isDTR()
	{
		return (out.isDTR(in.PortNumber));
	}

	public void setRTS(boolean rts)
	{
		out.setRTS(out.PortNumber, rts);
	}

	public boolean isRTS()
	{
		return (in.isRTS(in.PortNumber));
	}

	public boolean isCTS()
	{
		return (in.isCTS(in.PortNumber));
	}

	public boolean isDSR()
	{
		return (in.isDSR(in.PortNumber));
	}

	public boolean isRI()
	{
		return (in.isRI(in.PortNumber));
	}

	public boolean isCD()
	{
		return (in.isCD(out.PortNumber));
	}
	public boolean isFrame()
	{
		return(in.isFrame(in.PortNumber));
	}
	public boolean isEmpty()
	{
		return(in.isEmpty(in.PortNumber));
	}
	public boolean isOverrun()
	{
		return(in.isOverrun(in.PortNumber));
	}
	public boolean isBreak()
	{
		return(in.isBreak(in.PortNumber));
	}
	public boolean isParity()
	{
		return(in.isParity(in.PortNumber));
	}

	public void addEventListener(SerialPortEventListener lsnr) throws TooManyListenersException 
	{
		this.thread_die = false;
		this.SPEventListener = lsnr;
		RXMonitor rxmondata = new RXMonitor(in,this);
		/*endless loop that read port with blocking */
		rxmondata.start(); 
	}

	public void removeEventListener()
	{
		System.out.println("removelistener");
		if (this.SPEventListener != null) {
/*
			monitor = monitor & ~SerialPortEvent.DATA_AVAILABLE;
			monitor = monitor & ~SerialPortEvent.FE;
			cant assume we are not in the monitor logic in the
			thread....
	
*/
			monitor = 0;
/* TRENT
			this.SPEventListener = null;
*/
			this.run_montitor = false;
/*
   			ugly problems caused here.  jarvi@ezlink.com
			The thread is going to die if this.thread_die is true.  
			now the thread is gone and we try...

			rxmondata.stop();

			I'm not even sure we dont want suspend/resume.
   
*/
		} 
	}

	public void notifyOnDataAvailable(boolean enable)  
	{
		if (enable)
			monitor = monitor | SerialPortEvent.DATA_AVAILABLE;
		else
			monitor = monitor & ~SerialPortEvent.DATA_AVAILABLE;
	}

	public void notifyOnOutputEmpty(boolean enable)
	{
		if (enable)
			monitor = monitor | SerialPortEvent.OUTPUT_BUFFER_EMPTY;
		else
			monitor = monitor & ~SerialPortEvent.OUTPUT_BUFFER_EMPTY;
	}

	public void notifyOnCTS(boolean enable)
	{
		if (enable)
			monitor = monitor | SerialPortEvent.CTS;
		else
			monitor = monitor & ~SerialPortEvent.CTS;

	}

	public void notifyOnDSR(boolean enable)
	{
		if (enable)
			monitor = monitor | SerialPortEvent.DSR;
		else
			monitor = monitor & ~SerialPortEvent.DSR;
	}

	public void notifyOnRingIndicator(boolean enable)  
	{
		if (enable)
			monitor = monitor | SerialPortEvent.RI;
		else
			monitor = monitor & ~SerialPortEvent.RI;

	}

	public void notifyOnCarrierDetect(boolean enable)
	{
		if (enable)
			monitor = monitor | SerialPortEvent.CD;
		else
			monitor = monitor & ~SerialPortEvent.CD;
	}

	public void notifyOnOverrunError(boolean enable)
	{
		if (enable)
			monitor = monitor | SerialPortEvent.OE;
		else
			monitor = monitor & ~SerialPortEvent.OE;
	}

	public void notifyOnParityError(boolean enable)
	{
		if (enable)
			monitor = monitor | SerialPortEvent.PE;
		else
			monitor = monitor & ~SerialPortEvent.PE;
	}

	public void notifyOnFramingError(boolean enable)
	{
		if (enable)
			monitor = monitor | SerialPortEvent.FE;
		else
			monitor = monitor & ~SerialPortEvent.FE;
	}

	public void notifyOnBreakInterrupt(boolean enable)
	{
		if (enable)
			monitor = monitor | SerialPortEvent.BI;
		else
			monitor = monitor & ~SerialPortEvent.BI;
	}

	// Nonstandard public functions

	/**
	   A query function to provide runtime version checking
	*/
	public static String getVersion()
	{
		return RXTXCommDriver.getVersion();
	}


	private SerialInputStream in;
	private SerialOutputStream out;

	private int databits = DATABITS_8;
	private int stopbits = STOPBITS_1;
	private int parity = PARITY_NONE;
	protected int flowmode = FLOWCONTROL_NONE;

	protected SerialPortEventListener SPEventListener;
	private RXMonitor rxmondata;      // thread to monitor data
	protected int monitor;            // characteristics to monitor
	protected boolean thread_die;     // flag used to kill threads
	protected boolean run_montitor;   // flag used to stop threads

	protected Termios out_termios; 
	protected Termios in_termios;

    // parameters for setting blocking characters
	private final static int BLOCK = 0;
	private final static int NOBLOCK  = 1;

	private boolean noblock = false;

}
